#!/usr/bin/env perl

package main v0.1.0;

use Pcore { SERVICE_NAME => 'selenium' };
use Pcore::Selenium;
use Pcore::App::Selenium;

Pcore::App::Selenium->new->run;

1;
__END__
=pod

=encoding utf8

=head1 REQUIRED ARGUMENTS

=over

=back

=head1 OPTIONS

=over

=back

=cut
